@@ -9,9 +9,11 @@
 """
 from __future__ import with_statement
 
+import re
 import os
 import sys
-from six import PY3, print_, exec_, b
+import types
+from six import PY3, print_, exec_, b, callable
 from glob import glob
 import optparse
 import textwrap
@@ -95,27 +97,67 @@ else:
     convert_with_2to3 = None
 
 
-def command(func):
-    name = func.__name__.replace('_', '-')
-    commands[name] = (func, [l.strip()
-                             for l in func.__doc__.splitlines()
-                             if l][0])
+class Command(object):
 
-    options = OrderedDict()
-    if PY3:
-        argcount = func.__code__.co_argcount
-        varnames = func.__code__.co_varnames[:argcount]
-        defaults = list(func.__defaults__ or [])
-    else:
-        argcount = func.func_code.co_argcount
-        varnames = func.func_code.co_varnames[:argcount]
-        defaults = list(func.func_defaults or [])
-    defaults = [required] * (argcount - len(defaults)) + defaults
-    for name, val in zip(varnames, defaults):
-        options[name] = val
+    @classmethod
+    def register(cls, *args, **kw):
+        if (not kw
+           and len(args) == 1
+           and isinstance(args[0], types.FunctionType)):
+            return cls._register(args[0])
+        else:
+            def _wrapper(func):
+                return cls._register(func, *args, **kw)
+            return _wrapper
 
-    func.options = options
-    return func
+    @classmethod
+    def _register(cls, func, call_prehook=None):
+        options = OrderedDict()
+        if PY3:
+            argcount = func.__code__.co_argcount
+            varnames = func.__code__.co_varnames[:argcount]
+            defaults = list(func.__defaults__ or [])
+        else:
+            argcount = func.func_code.co_argcount
+            varnames = func.func_code.co_varnames[:argcount]
+            defaults = list(func.func_defaults or [])
+        defaults = [required] * (argcount - len(defaults)) + defaults
+        for name, val in zip(varnames, defaults):
+            options[name] = val
+
+        cmdobj = cls(
+            func,
+            options,
+            [l.strip() for l in func.__doc__.splitlines() if l][0]
+        )
+        cmdobj.call_prehook = call_prehook
+
+        name = func.__name__.replace('_', '-')
+        commands[name] = cmdobj
+        func.cmdobj = cmdobj
+        return func
+
+    def __init__(self, function, options, description):
+        self.function = function
+        self.description = description
+        self.options = options
+
+        self.call_prehook = None
+
+    def __call__(self, options):
+        kwargs = self.options.copy()
+        if callable(self.call_prehook):
+            kwargs, options = self.call_prehook(self, kwargs, options)
+        for k, v in kwargs.items():
+            if v is required and not getattr(options, k):
+                msg = "'--%s' option is required." % k.replace('_', '-')
+                raise RuntimeError(msg)
+            kwargs[k] = getattr(options, k, v)
+
+        return self.function(**kwargs)
+
+
+command = Command.register
 
 
 def execfile_(filepath, _globals):
@@ -295,9 +337,9 @@ def stat(locale_dir, language=(), out=sys.stdout):
                        ("%d translated, "
                         "%d fuzzy, "
                         "%d untranslated." % (
-                        len(po.translated_entries()),
-                        len(po.fuzzy_entries()),
-                        len(po.untranslated_entries()),
+                            len(po.translated_entries()),
+                            len(po.fuzzy_entries()),
+                            len(po.untranslated_entries()),
                         )),
                        file=out)
 
@@ -352,7 +394,20 @@ def create_txconfig(out=sys.stdout):
     print_('Create:', target, file=out)
 
 
-@command
+def load_txconfig_project_name(cmdobj, required_options, options):
+    target = os.path.normpath('.tx/config')
+    if os.path.exists(target):
+        matched = re.search(r'\[(.*)\..*\]', open(target, 'r').read())
+        if matched:
+            options.transifex_project_name = matched.groups()[0]
+            print_(
+                'Project name loaded from .tx/config:',
+                options.transifex_project_name)
+
+    return required_options, options
+
+
+@command(call_prehook=load_txconfig_project_name)
 def update_txconfig_resources(transifex_project_name, locale_dir,
                               pot_dir=None, out=sys.stdout):
     """
@@ -396,8 +451,8 @@ def update_txconfig_resources(transifex_project_name, locale_dir,
             resource_path = relpath(base, pot_dir)
             pot = polib.pofile(pot_file)
             if len(pot):
-                resource_name = \
-                    resource_path.replace('\\', '/').replace('/', '--')
+                resource_name = re.sub(r'[\\/]', '--', resource_path)
+                resource_name = resource_name.replace('.', '_')
                 args = (args_tmpl % locals()).split()
                 txclib.utils.exec_command('set', args, tx_root)
             else:
@@ -415,7 +470,7 @@ def parse_option(argv):
 
     Environment Variables:
       All command-line options can be set with environment variables using the
-      format TXUTIL_<UPPER_LONG_NAME> . Dashes (-) have to replaced with
+      format SPHINXINTL_<UPPER_LONG_NAME> . Dashes (-) have to replaced with
       underscores (_).
 
       For example, to set the languages:
@@ -427,11 +482,13 @@ def parse_option(argv):
          sphinx-intl --language=de --language=ja <command>
     """) % {
         'commands': '\n  '.join(
-            '%s: %s' % (c, d) for c, (f, d) in commands.items())}
+            '%s: %s' % (name, cmdobj.description)
+            for name, cmdobj in commands.items()
+        )}
 
     parser = optparse.OptionParser(usage=usage)
     parser.add_option('-c', '--config', dest='config',
-                      type='string', action='store', default='conf.py',
+                      type='string', action='store', default=None,
                       metavar='FILE',
                       help='read configurations from FILE')
     parser.add_option('-l', '--language', dest='language',
@@ -480,6 +537,9 @@ def parse_option(argv):
                 else:
                     setattr(options, optname, value)
 
+    if options.config is None:
+        options.config = 'conf.py'
+
     if not options.locale_dir:
         config = read_config(options.config)
         if 'locale_dirs' not in config:
@@ -497,14 +557,7 @@ def run(argv):
     cmd = args[0]
 
     if cmd in commands:
-        func, desc = commands[cmd]
-        kwargs = func.options.copy()
-        for k, v in kwargs.items():
-            if v is required and not getattr(options, k):
-                msg = "'--%s' option is required." % k.replace('_', '-')
-                raise RuntimeError(msg)
-            kwargs[k] = getattr(options, k, v)
-        func(**kwargs)
+        commands[cmd](options)
     else:
         msg = 'unknown command: %s' % cmd
         raise RuntimeError(msg)
